﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ImageLab.Filter
{
    public class Coordinate
    {
        public int X, Y;
        public Coordinate(int X, int Y)
        {
            this.X = X;
            this.Y = Y;
        }
        public Coordinate()
        {
            this.X = 0;
            this.Y = 0;
        }
        /// <summary>
        /// Noninclusive upper boundrary, so safe for use with arrays. 
        /// </summary>
        /// <param name="input"></param>
        /// <param name="min"></param>
        /// <param name="max"></param>
        /// <returns></returns>
        public static Coordinate Clamp(Coordinate input, Coordinate min, Coordinate max)
        {
            Coordinate output = new Coordinate(input.X, input.Y);

            if (input.X < min.X) output.X = min.X;
            else if (input.X >= max.X) output.X = max.X - 1;

            if (input.Y < min.Y) output.Y = min.Y;
            else if (input.Y >= max.Y) output.Y = max.Y - 1;

            return output;
        }
        public static bool Contains(Coordinate input, Coordinate min, Coordinate max)
        {
            bool output =
                input.X >= min.X &
                input.X < max.X &
                input.Y >= min.Y &
                input.Y < max.Y;

            return output;
        }

        public static readonly Coordinate Right = new Coordinate(1, 0);
        public static readonly Coordinate Left = new Coordinate(-1, 0);
        public static readonly Coordinate Up = new Coordinate(0, -1);
        public static readonly Coordinate Down = new Coordinate(0, 1);
        public static readonly Coordinate Zero = new Coordinate(0, 0);

        public static Coordinate operator +(Coordinate a, Coordinate b)
        {
            return new Coordinate(a.X + b.X, a.Y + b.Y);
        }
        public static Coordinate operator -(Coordinate a, Coordinate b)
        {
            return new Coordinate(a.X - b.X, a.Y - b.Y);
        }
        public static Coordinate operator *(Coordinate a, Coordinate b)
        {
            return new Coordinate(a.X * b.X, a.Y * b.Y);
        }
        public static Coordinate operator *(int a, Coordinate b)
        {
            return new Coordinate(a * b.X, a * b.Y);
        }


        public override bool Equals(object obj)
        {
            var other = obj as Coordinate;
            if (other != null)
            {
                return this.X == other.X && this.Y == other.Y;
            }
            return base.Equals(obj);
        }
        public override string ToString()
        {
            return "{" + X + ", " + Y + "}";
        }
    }

}
