﻿using Microsoft.ParallelArrays;
using F4PA = Microsoft.ParallelArrays.Float4ParallelArray;
using F4 = Microsoft.ParallelArrays.Float4;
using PA = Microsoft.ParallelArrays.ParallelArrays;
using System;

namespace ImageLab.Filter
{
    public static class DistanceWeightedMedian
    {
        public static F4PA DWMedian(F4PA input, params object[] parameters)
        {
            var radius = (int)parameters[0];
            var n_samples = (2 * radius + 1) * (2 * radius + 1);
            var n_half = (n_samples - 1) / 2 + 1;
            var pa_half = new F4PA(new Float4(n_half), input.Shape);

            var pa_min = new F4PA(new Float4(), input.Shape);
            var pa_max = new F4PA(new Float4(1), input.Shape);

            var pa_win = new F4PA[n_samples];
            var dist_inv = new F4PA[n_samples];
            var dist_inv_float = new float[n_samples];
            var dist_inv_sum = 0f;
            
            for (int x = -radius, i = 0; x <= radius; x++)
            {
                for (int y = -radius; y <= radius; y++, i++)
                {
                    pa_win[i] = PA.Shift(input, new int[] { x, y }); 
                    
                    if (x == 0 & y == 0) dist_inv_float[i] = 1.5f;
                    else dist_inv_float[i] = 1f / (float)Math.Sqrt(x * x + y * y);

                    dist_inv_sum += dist_inv_float[i];
                }
            }

            for (int i = 0; i < n_samples; i++)
                dist_inv_float[i] = n_samples * dist_inv_float[i] / dist_inv_sum;

            for (int i = 0; i < n_samples; i++)
                dist_inv[i] = new F4PA(new F4(dist_inv_float[i]), input.Shape);

            FilterCore.Sort(pa_win, dist_inv);

            for (int i_run = 0; i_run < 8; i_run++)
            {
                var pa_pivot = (pa_max + pa_min) / new F4(2);
                var pa_count = new F4PA(new F4(), input.Shape);

                for (int i = 0; i < pa_win.Length; i++)
                    pa_count += PA.Select(pa_win[i] - pa_pivot, dist_inv[i], new F4(0));

                var pa_diff = pa_count - pa_half;

                pa_min = PA.Select(pa_diff, pa_pivot, pa_min);
                pa_max = PA.Select(-pa_diff, pa_pivot, pa_max);
            }
            return pa_min;
        }
    }
}
