﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.ParallelArrays;

namespace ImageLab.Filter
{
    [StructLayout(LayoutKind.Explicit)]
    public struct EasyCaster
    {
        [FieldOffset(0)]
        private Byte[] bytes;
        [FieldOffset(0)]
        private int[] ints;
        [FieldOffset(0)]
        private float[] floats;
        [FieldOffset(0)]
        private Length length;

        public static int[] ToInt(byte[] bytes)
        {
            EasyCaster ec = new EasyCaster();
            ec.bytes = bytes;
            ec.length.value = bytes.Length / 4;
            return ec.ints;
        }
        public static byte[] ToByte(int[] ints)
        {
            EasyCaster ec = new EasyCaster();
            ec.ints = ints;
            ec.length.value = ints.Length * 4;
            return ec.bytes;
        }
        public static float[] ToFloat(byte[] bytes)
        {
            EasyCaster ec = new EasyCaster();
            ec.bytes = bytes;
            ec.length.value = bytes.Length / 4;
            return ec.floats;
        }
        public static byte[] ToByte(float[] floats)
        {
            EasyCaster ec = new EasyCaster();
            ec.floats = floats;
            ec.length.value = floats.Length * 4;
            return ec.bytes;
        }
        private class Length
        {
            public int value;
        }
    }
    [StructLayout(LayoutKind.Explicit)]
    public struct EasyCaster2D
    {
        [FieldOffset(0)]
        private byte[] bytes;
        [FieldOffset(0)]
        private int[,] ints2D;
        [FieldOffset(0)]
        private int[] ints;
        [FieldOffset(0)]
        private float[] floats;
        [FieldOffset(0)]
        private Float4[,] color;


        [FieldOffset(0)]
        private Length length;

        public static byte[] ToByte(Float4[,] input, int outLength)
        {
            EasyCaster2D ec = new EasyCaster2D();
            ec.color = input;
            ec.length.value = outLength + 4;

            return ec.bytes;
        }
        public static byte[] ToByte(int[,] input, int outLength)
        {
            EasyCaster2D ec = new EasyCaster2D();
            ec.ints2D = input;
            ec.length.value = outLength + 16;

            return ec.bytes;
        }
        public static float[] ToFloat(Float4[,] input, int outLength)
        {
            EasyCaster2D ec = new EasyCaster2D();
            ec.color = input;
            ec.length.value = outLength + 4;

            return ec.floats;
        }
        public static int[] ToInt(int[,] input, int outLength)
        {
            EasyCaster2D ec = new EasyCaster2D();
            ec.ints2D = input;
            ec.length.value = outLength + 4;

            return ec.ints;
        }
        public static void WriteHeader(byte[] bytes, int width, int height)
        {
            var k1 = BitConverter.GetBytes(width);
            var k2 = BitConverter.GetBytes(height);
            bytes[0] = k1[0];
            bytes[1] = k1[1];
            bytes[2] = k1[2];
            bytes[3] = k1[3];
            bytes[4] = k2[0];
            bytes[5] = k2[1];
            bytes[6] = k2[2];
            bytes[7] = k2[3];

            for (int i = 8; i < 16; i++)
                bytes[i] = 0;
        }
        private class Length
        {
            public int value;
        }
    }
    [StructLayout(LayoutKind.Explicit)]
    struct Union
    {
        [FieldOffset(0)]
        public int i;
        [FieldOffset(0)]
        public double d;
        [FieldOffset(0)]
        public char c;
        [FieldOffset(0)]
        public byte b1;
    }
}
