﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media.Imaging;

namespace ImageLab.Filter
{
    public class FilterWindow
    {
        public int Radius { get; set; }

        public Coordinate Position { get; set; }
        public Coordinate ImageSize { get; set; }

        public Coordinate TopLeft
        {
            get
            {
                return Coordinate.Clamp(
                    Position + Radius * (Coordinate.Left + Coordinate.Up),
                    Coordinate.Zero,
                    ImageSize);
            }
        }
        public Coordinate TopRight
        {
            get
            {
                return Coordinate.Clamp(
                    Position + Radius * (Coordinate.Right + Coordinate.Up),
                    Coordinate.Zero,
                    ImageSize);
            }
        }
        public Coordinate BottomLeft
        {
            get
            {
                return Coordinate.Clamp(
                    Position + Radius * (Coordinate.Left + Coordinate.Down),
                    Coordinate.Zero,
                    ImageSize);
            }
        }
        public Coordinate BottomRight
        {
            get
            {
                return Coordinate.Clamp(
                    Position + Radius * (Coordinate.Right + Coordinate.Down),
                    Coordinate.Zero,
                    ImageSize);
            }
        }

        public int Left
        {
            get
            {
                return Position.X - Radius;
            }
        }
        public int Right
        {
            get
            {
                return Position.X + Radius;
            }
        }
        public int Top
        {
            get
            {
                return Position.Y - Radius;
            }
        }
        public int Bottom
        {
            get
            {
                return Position.Y + Radius;
            }
        }

        public FilterWindow(int width, int height, int radius)
        {
            Position = new Coordinate();
            ImageSize = new Coordinate(width, height);

            Radius = radius;
        }

        public void Move(Coordinate input)
        {
            Position = Coordinate.Clamp(Position + input, Coordinate.Zero, ImageSize);
        }
        public void MoveToTop()
        {
            Position.Y = 0;
        }
        public void MoveToBottom()
        {
            Position.Y = ImageSize.Y - 1;
        }
        public void MoveToFirst()
        {
            Position.X = 0;
        }
        public void MoveToLast()
        {
            Position.X = ImageSize.X - 1;
        }

        public Coordinate GetRelative(int x, int y)
        {
            return GetRelative(new Coordinate(x, y));
        }
        public Coordinate GetRelative(Coordinate input)
        {
            return Coordinate.Clamp(input, Coordinate.Zero, ImageSize);
        }

        public int Clamp(int input, int min, int max)
        {
            int output = input;

            if (input < min) output = min;
            else if (input >= max) output = max - 1;

            return output;
        }
    }
}
