﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.ParallelArrays;
using IPA = Microsoft.ParallelArrays.IntParallelArray;

namespace ImageLab.Filter
{
    public class Histogram : ImageLab.Filter.IHistogram
    {
        public readonly int NUMBER_OF_LEVELS = byte.MaxValue + 1;

        public int[] Frequency { get; private set; }
        public int Samples { get; private set; }

        public byte Median
        {
            get
            {
                byte i = 0;
                var sum = 0;

                for (; i < NUMBER_OF_LEVELS; i++)
                {
                    sum += this[i];
                    if (sum >= Samples / 2f) break;
                }

                return i;
            }
        }

        public Histogram()
        {
            Frequency = new int[NUMBER_OF_LEVELS];
        }
        public Histogram(int levels)
        {
            NUMBER_OF_LEVELS = levels;
            Frequency = new int[NUMBER_OF_LEVELS];
        }

        public void Add(byte input)
        {
            this[input]++;
            Samples++;
        }
        public void Add(byte input, int number)
        {
            this[input] += number;
            Samples += number;
        }
        public void Remove(byte input)
        {
            this[input]--;
            if (this[input] < 0) throw new Exception("Frequency is lower than zero!");
            Samples--;
        }
        public void Remove(byte input, int number)
        {
            this[input] -= number;
            if (this[input] < 0) throw new Exception("Frequency is lower than zero!");
            Samples -= number;
        }
        public void Add(Histogram input)
        {
            for (int i = 0; i < Length; i++)
            {
                this[i] += input[i];
            }
            Samples += input.Samples;
        }
        public void Remove(Histogram input)
        {
            for (int i = 0; i < Length; i++)
            {
                this[i] -= input[i];
                if (this[i] < 0) throw new Exception("Frequency is lower than zero!");
            }
            Samples -= input.Samples;

        }
        public void Clear()
        {
            for (int i = 0; i < Length; i++)
            {
                this[i] = 0;
            }
            Samples = 0;
        }

        public override string ToString()
        {
            string output = "{";

            for (int i = 0; i < Length; i++)
            {
                var observed = this[i];
                for (int j = 0; j < observed; j++)
                {
                    output += i + ", ";
                }
            }
            output += "}";
            return output;
        }

        public static Histogram operator +(Histogram a, Histogram b)
        {
            //if (a.frequency.Length != b.frequency.Length) throw new Exception("The histograms are of different lenght.");
            var output = new Histogram();
            for (int i = 0; i < a.Length; i++)
            {
                output[i] = a[i] + b[i];
            }
            output.Samples = a.Samples + b.Samples;
            return output;
        }
        public static Histogram operator -(Histogram a, Histogram b)
        {
            //if (a.frequency.Length != b.frequency.Length) throw new Exception("The histograms are of different lenght.");
            var output = new Histogram();
            for (int i = 0; i < a.Length; i++)
            {
                output[i] = a[i] - b[i];
            }
            output.Samples = a.Samples - b.Samples;
            return output;
        }

        public int this[int index]
        {
            get { return Frequency[index]; }
            set { Frequency[index] = value; }
        }
        public int Length { get { return Frequency.Length; } }
    }
}
