﻿using System;
using System.Linq;
using Microsoft.ParallelArrays;

using F4PA = Microsoft.ParallelArrays.Float4ParallelArray;
using FPA = Microsoft.ParallelArrays.FloatParallelArray;
using IPA = Microsoft.ParallelArrays.IntParallelArray;
using PA = Microsoft.ParallelArrays.ParallelArrays;

namespace ImageLab.Filter
{
    public static class Transforms
    {

        public static F4PA Stretch(F4PA input, params object[] parameters)
        {
            var xwidth = (int)parameters[0];
            var xheight = (int)parameters[1];
            var x = new int[] { xwidth, xheight };

            return PA.Stretch(input, x);
        }
        public static F4PA Transpose(F4PA input, params object[] parameters)
        {
            return PA.Transpose(input);
        }
        public static F4PA BrightnessContrast(F4PA input, params object[] parameters)
        {
            var brightness = new Float4((float)parameters[0]);
            var contrast = new Float4((float)parameters[1]);

            return input * contrast + brightness;
        }
        public static F4PA GammaCorrection(F4PA input, params object[] parameters)
        {
            var gamma = (float)parameters[0];
            var pa_gamma = new FPA(gamma, input.Shape);

            return FilterCore.PowerF4PA(input, pa_gamma);
        }
        public static F4PA ToBlackWhite(F4PA input, params object[] parameters)
        {
            var treshold = new F4PA(
                new Float4((float)parameters[0]),
                input.Shape);

            var zero = new F4PA(new Float4(0), input.Shape);
            var one = new F4PA(new Float4(1), input.Shape);

            var average = PA.FromFloat4ParallelArray(input, 1);
            average += PA.FromFloat4ParallelArray(input, 2);
            average += PA.FromFloat4ParallelArray(input, 3);
            average /= 3;

            var average_f4 = PA.ToFloat4ParallelArray(average);

            return PA.Select(average_f4 - treshold, one, zero);
        }


    }
}
