﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.ParallelArrays;


using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;


namespace ImageLab
{
    /// <summary>
    /// Interaction logic for Graph.xaml
    /// </summary>
    public partial class Graph : System.Windows.Controls.UserControl
    {
        private Float4[] data;

        public Float4[] Data
        {
            get { return data; }
            set { data = value; }
        }
        private Chart chart;
        private ChartArea chartArea;

        public Graph()
        {
            InitializeComponent();
        }
        private void UserControl_Initialized(object sender, EventArgs e)
        {
            this.chart = new Chart();
            this.chartArea = new ChartArea();

            this.chartArea.AlignmentStyle = AreaAlignmentStyles.All;
            this.chartArea.AlignmentOrientation = AreaAlignmentOrientations.All;

            this.chartArea.AxisX.Minimum = 0;
            this.chartArea.AxisX.Maximum = 1;

            this.chartArea.AxisY.LabelStyle.Enabled = false;
            this.chartArea.AxisX.LabelStyle.Enabled = false;

            this.chart.ChartAreas.Add(this.chartArea);

            this.chart.Click += delegate
            {
                this.chartArea.AxisY.IsLogarithmic = !this.chartArea.AxisY.IsLogarithmic;
                chartArea.AxisY.Maximum = maxY;
                chartArea.AxisY.Minimum = this.chartArea.AxisY.IsLogarithmic ?
                    maxY / 100 : 0;
            };

        }

        private float maxY = 0f;
        public void Update()
        {
            chart.Series.Clear();
            if (data == null) return;

            var s1 = new Series();
            s1.BorderDashStyle = ChartDashStyle.Solid;
            s1.ChartType = SeriesChartType.FastLine;
            s1.Color = System.Drawing.Color.Red;
            s1.BorderWidth = 2;

            var s2 = new Series();
            s2.BorderDashStyle = ChartDashStyle.Solid;
            s2.ChartType = SeriesChartType.FastLine;
            s2.Color = System.Drawing.Color.Green;
            s2.BorderWidth = 2;

            var s3 = new Series();
            s3.BorderDashStyle = ChartDashStyle.Solid;
            s3.ChartType = SeriesChartType.FastLine;
            s3.Color = System.Drawing.Color.Blue;
            s3.BorderWidth = 2;

            maxY = 0.0f;
            var maxX = 0f;

            var i = 0;
            for (; i < data.Length; i++)
            {
                var dx = 1f / data.Length;

                if (data[i].f1 == 0) data[i].f1 = float.Epsilon;
                if (data[i].f2 == 0) data[i].f2 = float.Epsilon;
                if (data[i].f3 == 0) data[i].f3 = float.Epsilon;


                s1.Points.Add(new DataPoint(i * dx, data[i].f1));
                s2.Points.Add(new DataPoint(i * dx, data[i].f2));
                s3.Points.Add(new DataPoint(i * dx, data[i].f3));

                maxY = Math.Max(
                    Math.Max(data[i].f1, data[i].f2),
                    Math.Max(data[i].f3, maxY));

                maxX = i * dx;
            }

            chart.Series.Add(s1);
            chart.Series.Add(s2);
            chart.Series.Add(s3);

            chartArea.AxisY.Maximum = maxY;
            chartArea.AxisY.Minimum = this.chartArea.AxisY.IsLogarithmic ?
                maxY / 100 : 0;

            chart.Invalidate();
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // Create the interop host control.            
            System.Windows.Forms.Integration.WindowsFormsHost host =
                new System.Windows.Forms.Integration.WindowsFormsHost();

            // Assign the MaskedTextBox control as the host control's child.
            host.Child = chart;

            // Add the interop host control to the Grid
            // control's collection of child controls.
            this.panel.Children.Add(host);
        }


    }
}
